/*	Sample program for Olimex AVR-MT with AT90S2313 processor
 *	Hold Button1 to set Led On
 *	Hold Button2 to blink the Led
 *	Hold Button3 to activate Buzzer
 *	Press Button4 to set Led On
 *	Press Button5 to set Led Off
 *	Press Button6 to toggle the Led
 *	Compile with AVRStudio+WinAVR (gcc version 3.4.6)
 */

#define __AVR_AT90S2313__	1

#include "avr/io.h"

unsigned char B[7];
unsigned int i;

void PORT_Init()
{
	PORTB = 0b00000000;
	DDRB  = 0b00010000;		//set Buzzer as output (Bit4 = 1)

	PORTD = 0b00000000;
	DDRD  = 0b00001010;		//set Relay as output (Bit3 = 1), set TX as output(Bit1 = 1)
}

/*
	Button state check:
	set one of the pins (PORTB5, PORTB6 or PORTB7) as output 
	with high value and watch states of other two pins 
	accordingly schematic as follow:
*/

//Button1: PORTB5 output, check PINB6
unsigned char Button_1()
{
	PORTB = (PORTB & 0b00011111) | 0b00100000;
	DDRB  = (DDRB  & 0b00011111) | 0b00100000;

	if (PINB & 0b01000000)
		return 1;
	else
		return 0;
}

//Button2: PORTB6 output, check PINB7
unsigned char Button_2()
{
	PORTB = (PORTB & 0b00011111) | 0b01000000;
	DDRB  = (DDRB  & 0b00011111) | 0b01000000;

	if (PINB & 0b10000000)
		return 1;
	else
		return 0;
}

//Button3: PORTB7 output, check PINB5
unsigned char Button_3()
{
	PORTB = (PORTB & 0b00011111) | 0b10000000;
	DDRB  = (DDRB  & 0b00011111) | 0b10000000;

	if (PINB & 0b00100000)
		return 1;
	else
		return 0;
}

//Button4: PORTB6 output, check PINB5
unsigned char Button_4()
{
	PORTB = (PORTB & 0b00011111) | 0b01000000;
	DDRB  = (DDRB  & 0b00011111) | 0b01000000;

	if (PINB & 0b00100000)
		return 1;
	else
		return 0;
}

//Button5: PORTB7 output, check PINB6
unsigned char Button_5()
{
	PORTB = (PORTB & 0b00011111) | 0b10000000;
	DDRB  = (DDRB  & 0b00011111) | 0b10000000;

	if (PINB & 0b01000000)
		return 1;
	else
		return 0;
}

//Button1: PORTB6 output, check PINB7
unsigned char Button_6()
{
	PORTB = (PORTB & 0b00011111) | 0b00100000;
	DDRB  = (DDRB  & 0b00011111) | 0b00100000;

	if (PINB & 0b10000000)
		return 1;
	else
		return 0;
}

int main()
{
	//Value of B[i] indicates the state of the Button I
	//1 - Button is pressed; 0 - Button is released
	PORT_Init();
	for (i=0; i<=6; i++) B[i] = 0;
	while (1)
	{
		if (Button_1() & (!B[1]))
		{
			PORTD = PORTD | 0b00001000;
			B[1] = 1;			
		}

		if ((!Button_1()) & B[1])
		{
			PORTD = PORTD & 0b11110111;
			B[1] = 0;
		}
		
		if (Button_2())
		{
			if (PIND & 0b00001000)
				PORTD = PORTD & 0b11110111;
			else
				PORTD = PORTD | 0b00001000;
			for (i=65000; i; i--);
		}

		if (Button_3())
		{
			PORTB = PORTB | 0b00010000;
			for (i=500; i; i--);
			PORTB = PORTB & 0b11101111;
			for (i=500; i; i--);
		}

		if (Button_4() & (!B[4]))
		{
			PORTD = PORTD | 0b00001000;
			B[4] = 1;
		}

		if (!Button_4() & B[4])
		{
			B[4] = 0;
		}

		if (Button_5() & (!B[5]))
		{
			PORTD = PORTD & 0b11110111;
			B[5] = 1;
		}

		if (!Button_5() & B[5])
		{
			B[5] = 0;
		}

		if (Button_6() & (!B[6]))
		{
			if (PIND & 0b00001000)
				PORTD = PORTD & 0b11110111;
			else
				PORTD = PORTD | 0b00001000;
			B[6] = 1;
		}

		if (!Button_6() & B[6])
		{
			B[6] = 0;
		}
	}
	return 0;
}
